@@ -4,12 +4,12 @@
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.
-  
+
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
-  
+
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -32,6 +32,8 @@
 typedef struct _sen_set_element entry;
 typedef struct _sen_set_element_str entry_str;
 
+#define SEN_SET_STRHASH(e) (((entry_str *)(e))->key)
+
 inline static entry *
 entry_new(sen_set *set)
 {
@@ -39,6 +41,7 @@ entry_new(sen_set *set)
   if (set->garbages) {
     e = set->garbages;
     set->garbages = *((entry **)e);
+    memset(e, 0, set->entry_size);
   } else {
     byte *chunk = set->chunks[set->curr_chunk];
     if (!chunk) {
@@ -99,25 +102,25 @@ sen_set_reset(sen_set * set, uint32_t ne)
   if (!(index = SEN_CALLOC(n * sizeof(entry *)))) { return sen_memory_exhausted; }
   m = n - 1;
   if (set->key_size) {
-    for (j = set->max_offset + 1, sp = set->index; j; j--, sp++) { 
-      e = *sp; 
-      if (!e || (e == GARBAGE)) { continue; } 
-      for (i = e->key; ; i += 1048573) { 
-        dp = index + (i & m); 
-        if (!*dp) { break; } 
-      } 
-      *dp = e; 
-    } 
+    for (j = set->max_offset + 1, sp = set->index; j; j--, sp++) {
+      e = *sp;
+      if (!e || (e == GARBAGE)) { continue; }
+      for (i = e->key; ; i += 1048573) {
+        dp = index + (i & m);
+        if (!*dp) { break; }
+      }
+      *dp = e;
+    }
   } else {
-    for (j = set->max_offset + 1, sp = set->index; j; j--, sp++) { 
-      e = *sp; 
-      if (!e || (e == GARBAGE)) { continue; } 
-      for (i = ((entry_str *)e)->key; ; i += 1048573) { 
-        dp = index + (i & m); 
-        if (!*dp) { break; } 
-      } 
-      *dp = e; 
-    } 
+    for (j = set->max_offset + 1, sp = set->index; j; j--, sp++) {
+      e = *sp;
+      if (!e || (e == GARBAGE)) { continue; }
+      for (i = SEN_SET_STRHASH(e); ; i += 1048573) {
+        dp = index + (i & m);
+        if (!*dp) { break; }
+      }
+      *dp = e;
+    }
   }
   {
     entry **i0 = set->index;
@@ -199,7 +202,7 @@ sen_set_str_at(sen_set *set, const char *key, void **value)
   uint32_t h = str_hash((unsigned char *)key), i, m = set->max_offset;
   for (i = h; ep = index + (i & m), (e = *ep); i += 1048573) {
     if (e == GARBAGE) { continue; }
-    if (((entry_str *)e)->key == h && !strcmp(key, SEN_SET_STRKEY(e))) {
+    if (SEN_SET_STRHASH(e) == h && !strcmp(key, SEN_SET_STRKEY(e))) {
       if (value) { *value = SEN_SET_STRVAL(e); }
       return (sen_set_eh *) ep;
     }
@@ -274,7 +277,7 @@ sen_set_str_get(sen_set *set, const char *key, void **value)
     if (e == GARBAGE) {
       if (!np) { np = ep; }
     } else {
-      if (((entry_str *)e)->key == h && !strcmp(key, SEN_SET_STRKEY(e))) { goto exit; }
+      if (SEN_SET_STRHASH(e) == h && !strcmp(key, SEN_SET_STRKEY(e))) { goto exit; }
     }
   }
   if (np) {
@@ -282,13 +285,12 @@ sen_set_str_get(sen_set *set, const char *key, void **value)
     ep = np;
   }
   e = entry_new(set);
-  ((entry_str *)e)->key = h;
-  e->key = h;
+  SEN_SET_STRHASH(e) = h;
   SEN_SET_STRKEY(e) = SEN_STRDUP(key);
   *ep = e;
   set->n_entries++;
 exit :
-  if (value) { *value = e->dummy; }
+  if (value) { *value = SEN_SET_STRVAL(e); }
   return ep;
 }
 
@@ -348,7 +350,6 @@ sen_set_del(sen_set *set, sen_set_eh *ep)
   e = *ep;
   *ep = GARBAGE;
   if (!set->key_size) { SEN_FREE(SEN_SET_STRKEY(e)); }
-  //  memset(e, 0, set->entry_size);
   *((entry **)e) = set->garbages;
   set->garbages = e;
   set->n_entries--;
@@ -373,7 +374,11 @@ sen_set_cursor_next(sen_set_cursor *c, void **key, void **value)
 {
   uint32_t i;
   entry *e, **ep;
-  if (!c || !c->rest || c->index != c->set->index) { return NULL; }
+  if (!c || !c->rest) { return NULL; }
+  if (c->index != c->set->index) {
+    SEN_LOG(sen_log_alert, "sen_reset invoked during sen_set_cursor is opened!");
+    return NULL;
+  }
   for (i = c->rest, ep = c->curr;;i--, ep++) {
     if (!i) {
       c->rest = 0;
@@ -622,7 +627,7 @@ sen_set_sort(sen_set *set, int limit, sen_set_sort_optarg *optarg)
     } else if (optarg->compar_arg) {
       sort_int(set, res, limit, ((intptr_t)optarg->compar_arg) / sizeof(int32_t), dir);
       goto exit;
-    } 
+    }
   }
   switch (set->key_size) {
   case 0 :
@@ -787,6 +792,7 @@ sen_rc
 sen_set_array_init(sen_set *set, uint32_t size)
 {
   SEN_ASSERT(!set->n_entries);
+  SEN_ASSERT(!set->garbages);
   set->arrayp = 1;
   set->curr_entry = 0;
   if (set->chunks[SEN_SET_MAX_CHUNK]) {
